@@ -1,3 +1,4 @@
+from pathlib import Path
 import json
 import math
 import os
@@ -12,73 +13,43 @@ import numpy as np
 import openmc
 import plotly.graph_objects as go
 
+import neutronics_material_maker as nmm
+import dagmc_h5m_file_inspector as di
 
-def plotly_trace(
-    points: Union[List[Tuple[float, float]], List[Tuple[float, float, float]]],
-    mode: str = "markers+lines",
-    name: str = None,
-    color: Union[Tuple[float, float, float], Tuple[float, float, float, float]] = None,
-) -> Union[go.Scatter, go.Scatter3d]:
-    """Creates a plotly trace representation of the points of the Shape
-    object. This method is intended for internal use by Shape.export_html.
 
-    Args:
-        points: A list of tuples containing the X, Z points of to add to
-            the trace.
-        mode: The mode to use for the Plotly.Scatter graph. Options include
-            "markers", "lines" and "markers+lines". Defaults to
-            "markers+lines"
-        name: The name to use in the graph legend color
-
-    Returns:
-        plotly trace: trace object
-    """
-
-    if color is not None:
-        color_list = [i * 255 for i in color]
-
-        if len(color_list) == 3:
-            color = "rgb(" + str(color_list).strip("[]") + ")"
-        elif len(color_list) == 4:
-            color = "rgba(" + str(color_list).strip("[]") + ")"
-
-    if name is None:
-        name = "Shape not named"
+def create_material(material_tag: str, material_entry):
+    if isinstance(material_entry, str):
+        openmc_material = nmm.Material.from_library(
+            name=material_entry, material_id=None
+        ).openmc_material
+    elif isinstance(material_entry, openmc.Material):
+        # sets the material name in the event that it had not been set
+        openmc_material = material_entry
+    elif isinstance(material_entry, (nmm.Material)):
+        # sets the material tag in the event that it had not been set
+        openmc_material = material_entry.openmc_material
     else:
-        name = name
-
-    text_values = []
-
-    for i, point in enumerate(points):
-        text = "point number= {i} <br> x={point[0]} <br> y= {point[1]}"
-        if len(point) == 3:
-            text = text + "<br> z= {point[2]} <br>"
-
-        text_values.append(text)
-
-    if all(len(entry) == 3 for entry in points):
-        trace = go.Scatter3d(
-            x=[row[0] for row in points],
-            y=[row[1] for row in points],
-            z=[row[2] for row in points],
-            mode=mode,
-            marker={"size": 3, "color": color},
-            name=name,
+        raise TypeError(
+            "materials must be either a str, \
+            openmc.Material, nmm.MultiMaterial or nmm.Material object \
+            not a ",
+            type(material_entry),
+            material_entry,
         )
+    openmc_material.name = material_tag
+    return openmc_material
 
-        return trace
 
-    trace = go.Scatter(
-        x=[row[0] for row in points],
-        y=[row[1] for row in points],
-        hoverinfo="text",
-        text=text_values,
-        mode=mode,
-        marker={"size": 5, "color": color},
-        name=name,
-    )
+def create_openmc_materials(h5m_filename):
 
-    return trace
+    materials_in_h5m = di.get_materials_from_h5m(h5m_filename)
+    openmc_materials = {}
+    for material_tag in materials_in_h5m:
+        if material_tag != "graveyard":
+            openmc_material = create_material(material_tag, "Be")
+            openmc_materials[material_tag] = openmc_material
+
+    return openmc.Materials(list(openmc_materials.values()))
 
 
 def silently_remove_file(filename: str):
@@ -91,19 +62,19 @@ def silently_remove_file(filename: str):
         pass  # in some cases the file will not exist
 
 
-def _save_2d_mesh_tally_as_png(score: str, filename: str, tally) -> str:
+def _save_2d_mesh_tally_as_png(filename: str, tally) -> str:
     """Extracts 2D mesh tally results from a tally and saves the result as
     a png image.
 
     Arguments:
-        score (str): The tally score to filter the tally with, e.g. ‘flux’,
-            ‘heating’, etc.
+        score (str): The tally score to filter the tally with, e.g. 'flux',
+            'heating', etc.
         filename (str): The filename to use when saving the png output file
         tally (opencmc.tally()): The OpenMC to extract the mesh tally
-            resutls  from.
+            results from.
     """
 
-    my_slice = tally.get_slice(scores=[score])
+    my_slice = tally.get_slice(scores=tally.scores)
     tally_filter = tally.find_filter(filter_type=openmc.MeshFilter)
     shape = tally_filter.mesh.dimension.tolist()
     shape.remove(1)
@@ -209,11 +180,12 @@ def process_results(
 
             data_frame = tally.get_pandas_dataframe()
             tally_result = data_frame["mean"].sum()
-            tally_std_dev = data_frame["std. dev."].sum()
             results[tally.name] = {
                 "result": tally_result,
-                "std. dev.": tally_std_dev,
             }
+            if "std. dev." in data_frame.keys():
+                tally_std_dev = data_frame["std. dev."].sum()
+                results[tally.name]["std. dev."] = tally_std_dev
 
         elif tally.name.endswith("heating"):
 
@@ -249,20 +221,23 @@ def process_results(
 
             data_frame = tally.get_pandas_dataframe()
             tally_result = data_frame["mean"].sum()
-            tally_std_dev = data_frame["std. dev."].sum()
             results[tally.name]["fast flux per source particle"] = {
-                "result": tally_result,
-                "std. dev.": tally_std_dev,
+                "result": tally_result
             }
+            if "std. dev." in data_frame.keys():
+                tally_std_dev = data_frame["std. dev."].sum()
+                results[tally.name]["fast flux per source particle"][
+                    "std. dev."
+                ] = tally_std_dev
 
             if fusion_power is not None:
                 results[tally.name]["fast flux per second"] = {
-                    "result": tally_result * number_of_neutrons_per_second,
+                    "result": tally_result * number_of_neutrons_per_second
                 }
 
             if fusion_energy_per_pulse is not None:
                 results[tally.name]["fast flux per pulse"] = {
-                    "result": tally_result * number_of_neutrons_per_pulse,
+                    "result": tally_result * number_of_neutrons_per_pulse
                 }
 
         elif tally.name.endswith("flux"):
@@ -277,12 +252,12 @@ def process_results(
 
             if fusion_power is not None:
                 results[tally.name]["flux per second"] = {
-                    "result": tally_result * number_of_neutrons_per_second,
+                    "result": tally_result * number_of_neutrons_per_second
                 }
 
             if fusion_energy_per_pulse is not None:
                 results[tally.name]["flux per pulse"] = {
-                    "result": tally_result * number_of_neutrons_per_pulse,
+                    "result": tally_result * number_of_neutrons_per_pulse
                 }
 
         elif tally.name.endswith("spectra"):
@@ -339,18 +314,12 @@ def process_results(
                 }
 
         elif "_on_2D_mesh" in tally.name:
-            score = tally.name.split("_")[0]
+            # score = tally.name.split("_")[0]
             _save_2d_mesh_tally_as_png(
-                score=score,
-                tally=tally,
-                filename=tally.name.replace(
-                    "(",
-                    "").replace(
-                    ")",
-                    "").replace(
-                    ",",
-                    "-"),
-            )
+                tally=tally, filename=tally.name.replace(
+                    "(", "").replace(
+                    ")", "").replace(
+                    ",", "-"), )
 
         elif "_on_3D_mesh" in tally.name:
             print(f"processing {tally.name}")
@@ -517,122 +486,124 @@ def write_3d_mesh_tally_to_vtk(
     return outfile
 
 
-def create_initial_particles(
-        source,
-        number_of_source_particles: int = 2000) -> str:
-    """Accepts an openmc source and creates an initial_source.h5 that can be
-    used to find intial xyz, direction and energy of the partice source.
+# @shimwell are we keeping this?
+# def create_initial_particles(
+#         source,
+#         number_of_source_particles: int = 2000) -> str:
+#     """Accepts an openmc source and creates an initial_source.h5 that can be
+#     used to find intial xyz, direction and energy of the partice source.
 
-    Arguments:
-        source: (openmc.Source()): the OpenMC source to create an initial source
-            file from.
-        number_of_source_particles: The number of particle to sample.
+#     Arguments:
+#         source: (openmc.Source()): the OpenMC source to create an initial source
+#             file from.
+#         number_of_source_particles: The number of particle to sample.
 
-    Returns:
-        str: the filename of the h5 file produced
-    """
+#     Returns:
+#         str: the filename of the h5 file produced
+#     """
 
-    # MATERIALS
+#     # MATERIALS
 
-    # no real materials are needed for finding the source
-    mats = openmc.Materials([])
+#     # no real materials are needed for finding the source
+#     mats = openmc.Materials([])
 
-    # GEOMETRY
+#     # GEOMETRY
 
-    # just a minimal geometry
-    outer_surface = openmc.Sphere(r=100000, boundary_type="vacuum")
-    cell = openmc.Cell(region=-outer_surface)
-    universe = openmc.Universe(cells=[cell])
-    geom = openmc.Geometry(universe)
+#     # just a minimal geometry
+#     outer_surface = openmc.Sphere(r=100000, boundary_type="vacuum")
+#     cell = openmc.Cell(region=-outer_surface)
+#     universe = openmc.Universe(cells=[cell])
+#     geom = openmc.Geometry(universe)
 
-    # SIMULATION SETTINGS
+#     # SIMULATION SETTINGS
 
-    # Instantiate a Settings object
-    sett = openmc.Settings()
-    # this will fail but it will write the initial_source.h5 file first
-    sett.run_mode = "eigenvalue"
-    sett.particles = number_of_source_particles
-    sett.batches = 1
-    sett.inactive = 0
-    sett.write_initial_source = True
+#     # Instantiate a Settings object
+#     sett = openmc.Settings()
+#     # this will fail but it will write the initial_source.h5 file first
+#     sett.run_mode = "eigenvalue"
+#     sett.particles = number_of_source_particles
+#     sett.batches = 1
+#     sett.inactive = 0
+#     sett.write_initial_source = True
 
-    sett.source = source
+#     sett.source = source
 
-    model = openmc.model.Model(geom, mats, sett)
+#     model = openmc.model.Model(geom, mats, sett)
 
-    silently_remove_file("settings.xml")
-    silently_remove_file("materials.xml")
-    silently_remove_file("geometry.xml")
-    silently_remove_file("settings.xml")
-    silently_remove_file("tallies.xml")
-    model.export_to_xml()
+#     silently_remove_file("settings.xml")
+#     silently_remove_file("materials.xml")
+#     silently_remove_file("geometry.xml")
+#     silently_remove_file("settings.xml")
+#     silently_remove_file("tallies.xml")
+#     model.export_to_xml()
 
-    # this just adds write_initial_source == True to the settings.xml
-    tree = ET.parse("settings.xml")
-    root = tree.getroot()
-    elem = SubElement(root, "write_initial_source")
-    elem.text = "true"
-    tree.write("settings.xml")
+#     # this just adds write_initial_source == True to the settings.xml
+#     tree = ET.parse("settings.xml")
+#     root = tree.getroot()
+#     elem = SubElement(root, "write_initial_source")
+#     elem.text = "true"
+#     tree.write("settings.xml")
 
-    # This will crash hence the try except loop, but it writes the
-    # initial_source.h5
-    openmc.run(output=False)
-    try:
-        openmc.run(output=False)
-    except BaseException:
-        pass
+#     # This will crash hence the try except loop, but it writes the
+#     # initial_source.h5
+#     openmc.run(output=False)
+#     try:
+#         openmc.run(output=False)
+#     except BaseException:
+#         pass
 
-    return "initial_source.h5"
+#     return "initial_source.h5"
 
 
-def extract_points_from_initial_source(
-    input_filename: str = "initial_source.h5", view_plane: str = "RZ"
-) -> list:
-    """Reads in an initial source h5 file (generated by OpenMC), extracts point
-    and projects them onto a view plane.
+# @shimwell are we keeping this?
+# def extract_points_from_initial_source(
+#     input_filename: str = "initial_source.h5", view_plane: str = "RZ"
+# ) -> list:
+#     """Reads in an initial source h5 file (generated by OpenMC), extracts point
+#     and projects them onto a view plane.
 
-    Arguments:
-        input_filename: the OpenMC source to create an initial source
-            file from.
-        view_plane: The plane to project. Options are 'XZ', 'XY', 'YZ',
-            'YX', 'ZY', 'ZX', 'RZ' and 'XYZ'. Defaults to 'RZ'. Defaults to
-            'RZ'.
+#     Arguments:
+#         input_filename: the OpenMC source to create an initial source
+#             file from.
+#         view_plane: The plane to project. Options are 'XZ', 'XY', 'YZ',
+#             'YX', 'ZY', 'ZX', 'RZ' and 'XYZ'. Defaults to 'RZ'. Defaults to
+#             'RZ'.
 
-    Returns:
-        list: list of points extracted
-    """
-    import h5py
+#     Returns:
+#         list: list of points extracted
+#     """
+#     import h5py
 
-    h5_file = h5py.File(input_filename, "r")
-    dset = h5_file["source_bank"]
+#     h5_file = h5py.File(input_filename, "r")
+#     dset = h5_file["source_bank"]
 
-    points = []
+#     points = []
 
-    for particle in dset:
-        if view_plane == "XZ":
-            points.append((particle[0][0], particle[0][2]))
-        elif view_plane == "XY":
-            points.append((particle[0][0], particle[0][1]))
-        elif view_plane == "YZ":
-            points.append((particle[0][1], particle[0][2]))
-        elif view_plane == "YX":
-            points.append((particle[0][1], particle[0][0]))
-        elif view_plane == "ZY":
-            points.append((particle[0][2], particle[0][1]))
-        elif view_plane == "ZX":
-            points.append((particle[0][2], particle[0][0]))
-        elif view_plane == "RZ":
-            xy_coord = math.pow(particle[0][0], 2) + \
-                math.pow(particle[0][1], 2)
-            points.append((math.sqrt(xy_coord), particle[0][2]))
-        elif view_plane == "XYZ":
-            points.append((particle[0][0], particle[0][1], particle[0][2]))
-        else:
-            raise ValueError(
-                "view_plane value of ",
-                view_plane,
-                " is not supported")
-    return points
+#     for particle in dset:
+#         if view_plane == "XZ":
+#             points.append((particle[0][0], particle[0][2]))
+#         elif view_plane == "XY":
+#             points.append((particle[0][0], particle[0][1]))
+#         elif view_plane == "YZ":
+#             points.append((particle[0][1], particle[0][2]))
+#         elif view_plane == "YX":
+#             points.append((particle[0][1], particle[0][0]))
+#         elif view_plane == "ZY":
+#             points.append((particle[0][2], particle[0][1]))
+#         elif view_plane == "ZX":
+#             points.append((particle[0][2], particle[0][0]))
+#         elif view_plane == "RZ":
+#             xy_coord = math.pow(particle[0][0], 2) + \
+#                 math.pow(particle[0][1], 2)
+#             points.append((math.sqrt(xy_coord), particle[0][2]))
+#         elif view_plane == "XYZ":
+#             points.append((particle[0][0], particle[0][1], particle[0][2]))
+#         else:
+#             raise ValueError(
+#                 "view_plane value of ",
+#                 view_plane,
+#                 " is not supported")
+#     return points
 
 
 def diff_between_angles(angle_a: float, angle_b: float) -> float:
@@ -650,3 +621,57 @@ def diff_between_angles(angle_a: float, angle_b: float) -> float:
     if delta_mod > 180:
         delta_mod -= 360
     return delta_mod
+
+
+def find_bounding_box(h5m_filename: str) -> List[Tuple[float, float, float]]:
+    """Computes the bounding box of the DAGMC geometry
+
+    Args:
+        h5m_filename: the filename of the DAGMC h5m file
+
+    Returns:
+        x,y,z coordinates for the upper left and lower right corner
+    """
+    if not Path(h5m_filename).is_file:
+        msg = f"h5m file with filename {h5m_filename} not found"
+        raise FileNotFoundError(msg)
+    dag_univ = openmc.DAGMCUniverse(h5m_filename, auto_geom_ids=False)
+
+    geometry = openmc.Geometry(root=dag_univ)
+    geometry.root_universe = dag_univ
+    geometry.export_to_xml()
+
+    # exports materials.xml
+    # replace this with a empty materisl with the correct names
+    # self.create_openmc_materials()  # @shimwell do we need this?
+    # openmc.Materials().export_to_xml()
+    silently_remove_file("materials.xml")
+    materials = create_openmc_materials(h5m_filename)
+    materials.export_to_xml()
+
+    openmc.Plots().export_to_xml()
+
+    # a minimal settings .xml to allow openmc to init
+    settings = openmc.Settings()
+    settings.verbosity = 1
+    settings.batches = 1
+    settings.particles = 1
+    settings.export_to_xml()
+
+    # The -p runs in plotting mode which avoids the check that OpenMC does
+    # when looking for boundary surfaces and therefore avoids this error
+    # ERROR: No boundary conditions were applied to any surfaces!
+    openmc.lib.init(["-p"])
+
+    bbox = openmc.lib.global_bounding_box()
+    openmc.lib.finalize()
+
+    silently_remove_file("settings.xml")
+    silently_remove_file("plots.xml")
+    silently_remove_file("geometry.xml")
+    silently_remove_file("materials.xml")
+
+    return (
+        (bbox[0][0], bbox[0][1], bbox[0][2]),
+        (bbox[1][0], bbox[1][1], bbox[1][2]),
+    )
